"""Module containing the AGUIAgent class and supporting utilities."""

from agentpool.agents.agui_agent.agui_agent import AGUIAgent
from agentpool.agents.agui_agent.agui_converters import (
    ToolCallAccumulator,
    agui_to_native_event,
    to_agui_input_content,
    to_agui_tool,
)
from agentpool.agents.agui_agent.chunk_transformer import ChunkTransformer
from agentpool.agents.agui_agent.event_types import Event

__all__ = [
    # Main agent
    "AGUIAgent",
    # Chunk transformation
    "ChunkTransformer",
    # Extended event type
    "Event",
    # Converters
    "ToolCallAccumulator",
    "agui_to_native_event",
    "to_agui_input_content",
    "to_agui_tool",
]
