"""Runtime hook classes for agent lifecycle events."""

from __future__ import annotations

from agentpool.hooks.agent_hooks import AgentHooks
from agentpool.hooks.base import Hook, HookEvent, HookInput, HookResult
from agentpool.hooks.callable import CallableHook
from agentpool.hooks.command import CommandHook
from agentpool.hooks.prompt import PromptHook

__all__ = [
    "AgentHooks",
    "CallableHook",
    "CommandHook",
    "Hook",
    "HookEvent",
    "HookInput",
    "HookResult",
    "PromptHook",
]
